@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2022 Dirk-Jan C. Binnema <djcb.bulk@gmail.com>
+** Copyright (C) 2022-2023 Dirk-Jan C. Binnema <djcb.bulk@gmail.com>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -18,6 +18,7 @@
 */
 
 #include "mu-message-file.hh"
+#include "utils/mu-utils-file.hh"
 
 using namespace Mu;
 
@@ -27,21 +28,19 @@ Mu::maildir_from_path(const std::string& path, const std::string& root)
 	const auto pos = path.find(root);
 	if (pos != 0 || path[root.length()] != '/')
 		return Err(Error{Error::Code::InvalidArgument,
-				"root '%s' is not a root for path '%s'",
-				root.c_str(), path.c_str()});
+				"root '{}' is not a root for path '{}'", root, path});
 
 	auto mdir{path.substr(root.length())};
 	auto slash{mdir.rfind('/')};
 
 	if (G_UNLIKELY(slash == std::string::npos) || slash < 4)
 		return Err(Error{Error::Code::InvalidArgument,
-				"invalid path: %s", path.c_str()});
+				"invalid path: {}", path});
 	mdir.erase(slash);
 	auto subdir = mdir.data() + slash - 4;
 	if (G_UNLIKELY(strncmp(subdir, "/cur", 4) != 0 && strncmp(subdir, "/new", 4)))
 		return Err(Error::Code::InvalidArgument,
-			   "cannot find '/new' or '/cur' - invalid path: %s",
-			   path.c_str());
+			   "cannot find '/new' or '/cur' - invalid path: {}", path);
 
 	if (mdir.length() == 4)
 		return "/";
@@ -75,17 +74,11 @@ Mu::base_message_dir_file(const std::string& path)
 {
 	constexpr auto newdir{"/new"};
 
-	char *dirname{g_path_get_dirname(path.c_str())};
-	bool is_new{!!g_str_has_suffix(dirname, newdir)};
+	const auto dname{dirname(path)};
+	bool is_new{!!g_str_has_suffix(dname.c_str(), newdir)};
 
-	std::string mdir{dirname, ::strlen(dirname) - 4};
-	g_free(dirname);
-
-	char *basename{g_path_get_basename(path.c_str())};
-	std::string bname{basename};
-	g_free(basename);
-
-	return Ok(DirFile{std::move(mdir), std::move(bname), is_new});
+	std::string mdir{dname.substr(0, dname.size() - 4)};
+	return Ok(DirFile{std::move(mdir), basename(path), is_new});
 }
 
 Mu::Result<Mu::Flags>
@@ -109,7 +102,7 @@ Mu::flags_from_path(const std::string& path)
 	if (!flags) {
 		/* LCOV_EXCL_START*/
 		return Err(Error{Error::Code::InvalidArgument,
-				"invalid flags ('%s')", parts.flags_suffix.c_str()});
+				"invalid flags ('{}')", parts.flags_suffix});
 		/* LCOV_EXCL_STOP*/
 	}
 
@@ -118,8 +111,6 @@ Mu::flags_from_path(const std::string& path)
 }
 
 
-
-
 #ifdef BUILD_TESTS
 
 #include "utils/mu-test-utils.hh"
@@ -182,8 +173,8 @@ test_flags_from_path()
 		assert_valid_result(res);
 		/* LCOV_EXCL_START*/
 		if (g_test_verbose()) {
-			g_print("%s -> <%s>\n", tcase.first.c_str(),
-				to_string(res.value()).c_str());
+			mu_println("{} -> <{}>", tcase.first,
+				   to_string(res.value()));
 			g_assert_true(res.value() == tcase.second);
 		}
 		/*LCOV_EXCL_STOP*/
