@@ -29,6 +29,7 @@
 #include <atomic>
 #include <variant>
 #include <functional>
+#include <chrono>
 
 #include <cstring>
 #include <glib.h>
@@ -39,6 +40,7 @@
 #include "mu-query.hh"
 #include "mu-query-parser.hh"
 #include "mu-store.hh"
+#include "message/mu-labels.hh"
 
 #include "utils/mu-utils.hh"
 #include "utils/mu-utils-file.hh"
@@ -71,7 +73,7 @@ struct OutputStream {
 	 * @param cdr name of the output (e.g., "contacts")
 	 *
 	 * @return
-	 */
+`	 */
 	OutputStream(): out_{std::ostringstream{}} {}
 
 	/**
@@ -131,7 +133,8 @@ struct Server::Private {
 	{}
 
 	~Private() {
-		indexer().stop();
+		if (have_indexer_)
+			indexer().stop();
 		if (!tmp_dir_.empty())
 			remove_directory(tmp_dir_);
 	}
@@ -144,7 +147,7 @@ struct Server::Private {
 	// acccessors
 	Store&            store() { return store_; }
 	const Store&      store() const { return store_; }
-	Indexer&          indexer() { return store().indexer(); }
+	Indexer&          indexer() { have_indexer_ = true; return store().indexer(); }
 	void              do_index(const Indexer::Config& conf);
 	//CommandMap&       command_map() const { return command_map_; }
 
@@ -175,6 +178,7 @@ struct Server::Private {
 	void find_handler(const Command& cmd);
 	void help_handler(const Command& cmd);
 	void index_handler(const Command& cmd);
+	void label_handler(const Command& cmd);
 	void move_handler(const Command& cmd);
 	void mkdir_handler(const Command& cmd);
 	void ping_handler(const Command& cmd);
@@ -211,6 +215,8 @@ private:
 	const CommandHandler	command_handler_;
 	std::atomic<bool>       keep_going_{};
 	std::string		tmp_dir_;
+
+	bool have_indexer_{};
 };
 
 static void
@@ -345,8 +351,7 @@ Server::Private::make_command_map()
 	cmap.emplace(
 	    "index",
 	    CommandInfo{
-		ArgMap{{":my-addresses", ArgInfo{Type::List, false, "list of 'my' addresses"}},
-		       {":cleanup",
+		ArgMap{{":cleanup",
 			ArgInfo{Type::Symbol,
 				false,
 				"whether to remove stale messages from the store"}},
@@ -356,6 +361,18 @@ Server::Private::make_command_map()
 				"whether to avoid indexing up-to-date directories"}}},
 		"scan maildir for new/updated/removed messages",
 		[&](const auto& params) { index_handler(params); }});
+	cmap.emplace(
+	    "label",
+	    CommandInfo{
+		ArgMap{
+		    {":docid", ArgInfo{Type::Number, true, "document-id"}},
+		    {":labels", ArgInfo{Type::String, true,
+					    "label delta expressions (SPC-separated)"}},
+		    {":no-view",
+		     ArgInfo{Type::Symbol, false, "if set, do not hint at updating the view"}},
+		},
+		"move messages and/or change their flags",
+		[&](const auto& params) { label_handler(params); }});
 	cmap.emplace(
 	    "mkdir",
 	    CommandInfo{
@@ -548,19 +565,35 @@ Server::Private::contacts_handler(const Command& cmd)
 	mu_debug("sent {} of {} contact(s)", n, store().contacts_cache().size());
 }
 
+template<typename T, typename F>
+std::string quoted_join(const T& items, F&& func)
+{
+	std::vector<std::string> vec;
+
+	std::transform(items.begin(), items.end(), std::back_inserter(vec),
+		       [&](const auto& item) { return quote(func(item)); });
+
+	return join(vec,  " ");
+}
+
 void
 Server::Private::data_handler(const Command& cmd)
 {
 	const auto request_type{unwrap(cmd.symbol_arg(":kind"))};
 
-	if (request_type == "maildirs") {
-		auto&& out{make_output_stream()};
-		mu_print(out, "(");
-		for (auto&& mdir: store().maildirs())
-			mu_println(out, "{}", quote(std::move(mdir)));
-		mu_print(out, ")");
-		output(mu_format("(:maildirs {})", out.to_string()));
-	} else
+	if (request_type == "maildirs")
+		output(mu_format("(:maildirs ({}))",
+				 quoted_join(store().maildirs(),
+					     [](const auto& item) {
+					     return item; // identity
+					     })));
+	else if (request_type == "labels")
+		output(mu_format("(:labels ({}))",
+				 quoted_join(store().label_map(),
+					     [](const auto& item) {
+						     return item.first;
+					     })));
+	else
 		throw Error(Error::Code::InvalidArgument,
 			"invalid request type '{}'", request_type);
 }
@@ -640,61 +673,121 @@ Server::Private::output_results(const QueryResults& qres, size_t batch_size) con
 	return n;
 }
 
-void
-Server::Private::find_handler(const Command& cmd)
-{
-	const auto q{cmd.string_arg(":query").value_or("")};
-	const auto threads{cmd.boolean_arg(":threads")};
-	// perhaps let mu4e set this as frame-lines of the appropriate frame.
-	const auto batch_size{cmd.number_arg(":batch-size").value_or(200)};
-	const auto descending{cmd.boolean_arg(":descending")};
-	const auto maxnum{cmd.number_arg(":maxnum").value_or(-1) /*unlimited*/};
-	const auto skip_dups{cmd.boolean_arg(":skip-dups")};
-	const auto include_related{cmd.boolean_arg(":include-related")};
+struct FindProps {
+	std::string query;
+	int batch_size{200};
+	int maxnum{-1};
+	Field::Id sort_field_id{Field::Id::Date};
+	QueryFlags flags{QueryFlags::SkipUnreadable};
+};
+// XXX: once we move to C++20, use designated initializers
 
+
+static const std::pair<QueryFlags, std::string> flags_props[] = {
+	{ QueryFlags::Descending, ":descending" },
+	{ QueryFlags::SkipDuplicates, ":skip-dups" },
+	{ QueryFlags::IncludeRelated, ":include-related" },
+	{ QueryFlags::Threading, ":threads" }
+};
+
+/**
+ * Determine the find search properties from a command object
+ *
+ * @param cmd the command
+ *
+ * @return structure with find properties
+ */
+static FindProps
+determine_find_props(const Command& cmd)
+{
+	FindProps props{};
+
+	props.query = cmd.string_arg(":query").value_or("");
+	props.batch_size = cmd.number_arg(":batch-size").value_or(200);
+	if (props.batch_size < 1)
+		throw Error{Error::Code::InvalidArgument, "invalid batch-size {}", props.batch_size};
+
+	props.maxnum = cmd.number_arg(":maxnum").value_or(-1) /*unlimited*/;
+	if (props.maxnum < -1)
+		throw Error{Error::Code::InvalidArgument, "invalid max-num {}", props.maxnum};
+
+	const auto threads{cmd.boolean_arg(":threads")};
 	// complicated!
-	auto sort_field_id = std::invoke([&]()->Field::Id {
+	props.sort_field_id = std::invoke([&]()->Field::Id {
 		if (const auto arg = cmd.symbol_arg(":sortfield"); !arg)
 			return Field::Id::Date;
 		else if (arg->length() < 2)
-			throw Error{Error::Code::InvalidArgument, "invalid sort field '{}'",
+			throw Error{Error::Code::InvalidArgument, "invalid sort field parameter '{}'",
 				*arg};
 		else if (const auto field{field_from_name(arg->substr(1))}; !field)
+			// Note: mu4e gives us ':date' etc., hence the 'substr'
 			throw Error{Error::Code::InvalidArgument, "invalid sort field '{}'",
 				*arg};
+		else if (!field->is_sortable())
+			throw Error{Error::Code::InvalidArgument, "not a sortable field '{}'",
+				*arg};
+		else if (threads && field->id != Field::Id::Date)
+			throw Error{Error::Code::InvalidArgument,
+				     "with threads, you can only sort by date, not by '{}'", *arg};
 		else
 			return field->id;
 	});
 
-	if (batch_size < 1)
-		throw Error{Error::Code::InvalidArgument, "invalid batch-size {}", batch_size};
+	for (const auto& item: flags_props) {
+		if (cmd.boolean_arg(item.second))
+			props.flags |= item.first;
+	}
 
-	auto qflags{QueryFlags::SkipUnreadable}; // don't show unreadables.
+	return props;
+}
+
+/**
+ * Determine the result properties from the find results
+ *
+ * The result properties are passed back to the client
+ *
+ * @param props the find properties
+ *
+ * @return the result properties
+ */
+static Sexp
+determine_result_props(const FindProps& props, size_t found)
+{
 	Sexp resprops;
-	if (descending) {
-		qflags |= QueryFlags::Descending;
-		resprops.put_props(":reverse", Sexp::t_sym);
-	}
-	if (skip_dups) {
-		qflags |= QueryFlags::SkipDuplicates;
-		resprops.put_props(":skip-dups", Sexp::t_sym);
-	}
-	if (include_related) {
-		qflags |= QueryFlags::IncludeRelated;
-		resprops.put_props(":include-related", Sexp::t_sym);
-	}
-	if (threads) {
-		qflags |= QueryFlags::Threading;
-		resprops.put_props(":threads", Sexp::t_sym);
+
+	resprops.put_props(
+		":found", found,
+		":query", props.query,
+		":query-sexp", parse_query(props.query, false/*!expand*/),
+		":query-sexp-expanded", parse_query(props.query, true/*expand*/),
+		":maxnum", props.maxnum,
+		// mu4e expects ':'-prefixed sort-field, for historical reasons.
+		":sort-field", Sexp::Symbol{mu_format(":{}", field_from_id(props.sort_field_id).name)});
+
+	if (props.maxnum == -1)
+		resprops.put_props(":full", Sexp::t_sym);
+
+	for (const auto& item: flags_props) {
+		if (any_of(props.flags & item.first))
+			resprops.put_props(item.second, Sexp::t_sym);
 	}
 
+	return resprops;
+}
+
+void
+Server::Private::find_handler(const Command& cmd)
+{
+	const auto props = determine_find_props(cmd);
+
 	StopWatch sw{mu_format("{} (indexing: {})", __func__,
 			       indexer().is_running() ? "yes" :  "no")};
 
 	// we need to _lock_ the store while querying (which likely consists of
 	// multiple actual queries) + grabbing the results.
 	std::lock_guard l{store_.lock()};
-	auto qres{store_.run_query(q, sort_field_id, qflags, maxnum)};
+	auto qres{store_.run_query(props.query, props.sort_field_id,
+				   props.flags, props.maxnum)};
 	if (!qres)
 		throw Error(Error::Code::Query, "failed to run query: {}", qres.error().what());
 
@@ -702,15 +795,10 @@ Server::Private::find_handler(const Command& cmd)
 	 * knows it should erase the headers buffer. this will ensure that the
 	 * output of two finds will not be mixed. */
 	output_sexp(Sexp().put_props(":erase", Sexp::t_sym));
-	const auto bsize{static_cast<size_t>(batch_size)};
+	const auto bsize{static_cast<size_t>(props.batch_size)};
 	const auto foundnum = output_results(*qres, bsize);
 
-	output_sexp(resprops.put_props(
-			    ":found", foundnum,
-			    ":query", q,
-			    ":query-sexp", parse_query(q, false/*!expand*/).to_string(),
-			    ":query-sexp-expanded", parse_query(q, true/*expand*/).to_string(),
-			    ":maxnum", maxnum));
+	output_sexp(determine_result_props(props, foundnum));
 }
 
 void
@@ -774,10 +862,15 @@ Server::Private::do_index(const Indexer::Config& conf)
 {
 	StopWatch sw{"indexing"};
 	indexer().start(conf);
+
+	using namespace std::chrono_literals;
+	auto sleep_duration{125ms};
 	while (indexer().is_running()) {
-		std::this_thread::sleep_for(std::chrono::milliseconds(2000));
+		std::this_thread::sleep_for(sleep_duration);
 		output_sexp(get_stats(indexer().progress(), "running"),
 			    Server::OutputFlags::Flush);
+		// Exponential backoff: doubling sleep duration up to 2s
+		sleep_duration = std::min(sleep_duration * 2, 2000ms);
 	}
 	output_sexp(get_stats(indexer().progress(), "complete"),
 		    Server::OutputFlags::Flush);
@@ -796,6 +889,40 @@ Server::Private::index_handler(const Command& cmd)
 		throw Error{Error::Code::Xapian, "indexer is already running"};
 
 	do_index(conf);
+	store().serialize();
+}
+
+void
+Server::Private::label_handler(const Command& cmd)
+{
+	const auto labels{*cmd.string_arg(":labels")};
+	const auto docid{*cmd.number_arg(":docid")};
+	const auto no_view{cmd.boolean_arg(":noupdate")};
+
+	auto msg = store().find_message(docid)
+		.or_else([&]{throw Error{Error::Code::InvalidArgument,
+					"cannot find message {}", docid};}).value();
+
+	// special case: '-*' means "clear all labels"
+	if (labels == "-*") {
+		mu_debug("clear all labels from {} ({})", docid, join(msg.labels(), " "));
+		if (const auto res = store().clear_labels(msg); !res)
+			throw res.error();
+	} else {
+		const auto delta_labels = Labels::parse_delta_labels(labels, " ");
+		if (!delta_labels)
+			throw delta_labels.error();
+
+		mu_debug("apply {} to {} ({})", labels, docid, join(msg.labels(), " "));
+		if (const auto res = store().update_labels(msg, *delta_labels); !res)
+			throw res.error();
+	}
+
+	auto sexpstr = "(:update " + msg_sexp_str(msg, docid, {});
+	if (!no_view)
+		sexpstr += " :maybe-view t";
+	sexpstr += ')';
+	output(std::move(sexpstr));
 }
 
 void
@@ -911,7 +1038,7 @@ Server::Private::move_handler(const Command& cmd)
 	const auto flagopt{cmd.string_arg(":flags")};
 	const auto rename{cmd.boolean_arg(":rename")};
 	const auto no_view{cmd.boolean_arg(":noupdate")};
-	const auto docids{determine_docids(store_, cmd)};
+	const auto docids{determine_docids(store(), cmd)};
 
 	if (docids.empty()) {
 		if (!!cmd.string_arg(":msgid")) {
@@ -961,15 +1088,19 @@ Server::Private::ping_handler(const Command& cmd)
 	for (auto&& addr : store().config().get<Config::Id::PersonalAddresses>())
 		addrs.add(addr);
 
-	output_sexp(Sexp()
-		    .put_props(":pong", "mu")
-		    .put_props(":props",
-			       Sexp().put_props(
-				       ":version", VERSION,
-				       ":personal-addresses", std::move(addrs),
-				       ":database-path", store().path(),
-				       ":root-maildir", store().root_maildir(),
-				       ":doccount", storecount)));
+	Sexp props =  Sexp().put_props(
+		":version", VERSION,
+		":personal-addresses", std::move(addrs),
+		":database-path", store().path(),
+		":root-maildir", store().root_maildir(),
+		":doccount", storecount);
+
+	if (!options_.socket_path.empty())
+		props.put_props(
+			":scm-socket-path", options_.socket_path);
+
+	output_sexp(Sexp().put_props(":pong", "mu")
+		    .put_props(":props", std::move(props)));
 }
 
 void
@@ -1047,7 +1178,9 @@ Server::Private::view_mark_as_read(Store::Id docid, Message&& msg, bool rename)
 	if (rename)
 		move_opts |= Store::MoveOptions::ChangeName;
 
-	const auto ids{Store::id_vec(unwrap(store().move_message(docid, {}, nflags, move_opts)))};
+	const auto ids{Store::id_vec(
+			unwrap(store().move_message(docid, {},
+						    nflags, move_opts)))};
 	for (auto&& [id, moved_msg]: store().find_messages(ids))
 		output(mu_format("({} {})", id == docid ? ":view" : ":update",
 				 msg_sexp_str(moved_msg, id, {})));
@@ -1082,7 +1215,10 @@ Server::Private::view_handler(const Command& cmd)
 
 Server::Server(Store& store, const Server::Options& opts, Server::Output output)
     : priv_{std::make_unique<Private>(store, opts, output)}
-{}
+{
+	mu_message("created server with store @ {}; maildir @ {}",
+		   store.path(), store.root_maildir());
+}
 
 Server::~Server() = default;
 
