@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2021-2024 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2021-2025 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -35,6 +35,7 @@
 #include "mu-query.hh"
 #include "mu-xapian-db.hh"
 #include "mu-scanner.hh"
+#include "mu-store-labels.hh"
 
 #include "utils/mu-error.hh"
 
@@ -65,6 +66,7 @@ struct Store::Private {
 				    : XapianDb::Flavor::Open)},
 		config_{xapian_db_},
 		contacts_cache_{config_},
+		labels_cache_{config_},
 		root_maildir_{remove_slash(config_.get<Config::Id::RootMaildir>())},
 		message_opts_{make_message_options(config_)}
 		{}
@@ -74,17 +76,16 @@ struct Store::Private {
 		xapian_db_{XapianDb(path, XapianDb::Flavor::CreateOverwrite)},
 		config_{make_config(xapian_db_, root_maildir, conf)},
 		contacts_cache_{config_},
+		labels_cache_{config_},
 		root_maildir_{remove_slash(config_.get<Config::Id::RootMaildir>())},
 		message_opts_{make_message_options(config_)} {
-		// so tell xapian-db to update its internal cacheed values from
+		// so tell xapian-db to update its internal cached values from
 		// config. In practice: batch-size.
 		xapian_db_.reinit();
 	}
 
 	~Private() try {
 		mu_debug("closing store @ {}", xapian_db_.path());
-		if (!xapian_db_.read_only())
-			contacts_cache_.serialize();
 	} catch (...) {
 		mu_critical("caught exception in store dtor");
 	}
@@ -114,15 +115,16 @@ struct Store::Private {
 			return Message::Options::None;
 	}
 
+	Result<void> serialize();
+
 	Option<Message> find_message_unlocked(Store::Id docid) const;
+
+	using IdDocument = std::pair<Store::Id, Xapian::Document>;
+	Option<IdDocument> find_document_unlocked(const std::string& path) const;
+
 	Store::IdVec find_duplicates_unlocked(const Store& store,
 					      const std::string& message_id) const;
 
-	Result<Store::Id> add_message_unlocked(Message& msg);
-	Result<Store::Id> update_message_unlocked(Message& msg, Store::Id docid);
-	Result<Store::Id> update_message_unlocked(Message& msg, const std::string& old_path);
-
-
 	using PathMessage = std::pair<std::string, Message>;
 	Result<PathMessage> move_message_unlocked(Message&& msg,
 						  Option<const std::string&> target_mdir,
@@ -131,6 +133,7 @@ struct Store::Private {
 	XapianDb xapian_db_;
 	Config config_;
 	ContactsCache            contacts_cache_;
+	LabelsCache labels_cache_;
 	std::unique_ptr<Indexer> indexer_;
 
 	const std::string	root_maildir_;
@@ -140,33 +143,18 @@ struct Store::Private {
 };
 
 
-Result<Store::Id>
-Store::Private::add_message_unlocked(Message& msg)
+Result<void>
+Store::Private::serialize()
 {
-	auto&& docid{xapian_db_.add_document(msg.document().xapian_document())};
-	if (docid)
-		mu_debug("added message @ {}; docid = {}", msg.path(), *docid);
+	if (xapian_db_.read_only())
+		return Err(Error::Code::Store, "store must be writable");
 
-	return docid;
-}
+	mu_debug("serialize data into store @ {}", xapian_db_.path());
 
+	contacts_cache_.serialize(); // does the 'dirty' check internally.
+	labels_cache_.serialize();
 
-Result<Store::Id>
-Store::Private::update_message_unlocked(Message& msg, Store::Id docid)
-{
-	auto&& res{xapian_db_.replace_document(docid, msg.document().xapian_document())};
-	if (res)
-		mu_debug("updated message @ {}; docid = {}", msg.path(), *res);
-
-	return res;
-}
-
-Result<Store::Id>
-Store::Private::update_message_unlocked(Message& msg, const std::string& path_to_replace)
-{
-	return xapian_db_.replace_document(
-		field_from_id(Field::Id::Path).xapian_term(path_to_replace),
-		msg.document().xapian_document());
+	return Ok();
 }
 
 Option<Message>
@@ -180,6 +168,23 @@ Store::Private::find_message_unlocked(Store::Id docid) const
 		return Some(std::move(*msg));
 }
 
+
+Option<Store::Private::IdDocument>
+Store::Private::find_document_unlocked(const std::string& path) const
+{
+	constexpr auto path_field{field_from_id(Field::Id::Path)};
+
+	auto enq{xapian_db_.enquire()};
+	enq.set_query(Xapian::Query{path_field.xapian_term(path)});
+
+	if (auto mset{enq.get_mset(0, 1)}; mset.empty())
+		return Nothing; // message not found
+	else
+		return Some(IdDocument{*mset.begin(), mset.begin().get_document()});
+}
+
+
+
 Store::IdVec
 Store::Private::find_duplicates_unlocked(const Store& store,
 					 const std::string& message_id) const
@@ -189,7 +194,7 @@ Store::Private::find_duplicates_unlocked(const Store& store,
 		return {};
 	}
 
-	auto expr{mu_format("{}:{}",
+	const auto expr{mu_format("{}:{}",
 			    field_from_id(Field::Id::MessageId).shortcut,
 			    message_id)};
 	if (auto&& res{store.run_query(expr)}; !res) {
@@ -205,9 +210,29 @@ Store::Private::find_duplicates_unlocked(const Store& store,
 	}
 }
 
+static void
+reinit_export_labels(const Store& store)
+{
+	// slightly hacky way to get the cache-path...
+	const auto cache_path{canonicalize_filename(
+		join_paths(store.path(), "..")) + "/"};
+
+	if (const auto res =
+	    Mu::export_labels(store, "", cache_path); !res)
+		throw Mu::Error(Error::Code::CannotReinit,
+				"cannot re-init; "
+				"failed to export labels: {}",
+				res.error().what())
+			.add_hint("see mu-init(1) for details");
+	else {
+		mu_info("exported labels to: {}", *res);
+		mu_println("exported labels to: {}", *res);
+	}
+}
 
 Store::Store(const std::string& path, Store::Options opts)
-    : priv_{std::make_unique<Private>(path, none_of(opts & Store::Options::Writable))}
+    : priv_{std::make_unique<Private>(
+		path, none_of(opts & Store::Options::Writable))}
 {
 	if (none_of(opts & Store::Options::Writable) &&
 	    any_of(opts & Store::Options::ReInit))
@@ -216,12 +241,19 @@ Store::Store(const std::string& path, Store::Options opts)
 
 	const auto s_version{config().get<Config::Id::SchemaVersion>()};
 	if (any_of(opts & Store::Options::ReInit)) {
-		/* don't try to recover from version with an incompatible scheme */
+
+		/* export labels, if necessary (throws if there's an error) */
+		if (!label_map().empty())
+			reinit_export_labels(*this);
+
+		/* don't try to recover from version with an incompatible
+		 * scheme */
 		if (s_version < 500)
-			throw Mu::Error(Error::Code::CannotReinit,
-					"old schema ({}) is too old to re-initialize from",
-					s_version).add_hint("Invoke 'mu init' without '--reinit'; "
-							    "see mu-init(1) for details");
+			throw Mu::Error(
+				Error::Code::CannotReinit,
+				"old schema ({}) is too old to re-initialize from",
+				s_version).add_hint("Invoke 'mu init' without '--reinit'; "
+						    "see mu-init(1) for details");
 		const auto old_root_maildir{root_maildir()};
 
 		MemDb mem_db;
@@ -303,6 +335,13 @@ Store::contacts_cache() const
 	return priv_->contacts_cache_;
 }
 
+Result<void>
+Store::serialize()
+{
+	std::lock_guard guard{priv_->lock_};
+	return priv_->serialize();
+}
+
 Indexer&
 Store::indexer()
 {
@@ -344,8 +383,8 @@ Store::add_message(Message& msg, bool is_new)
 
 	std::lock_guard guard{priv_->lock_};
 	auto&& res = is_new ?
-		priv_->add_message_unlocked(msg) :
-		priv_->update_message_unlocked(msg, msg.path());
+		xapian_db().add_document(msg.document().xapian_document()) :
+		xapian_db().replace_document(msg.xapian_term(), msg.document().xapian_document());
 	if (!res)
 		return Err(res.error());
 
@@ -364,17 +403,24 @@ Store::add_message(const std::string& path, bool is_new)
 		return add_message(msg.value(), is_new);
 }
 
-
 bool
 Store::remove_message(const std::string& path)
 {
 	const auto term{field_from_id(Field::Id::Path).xapian_term(path)};
 
 	std::lock_guard guard{priv_->lock_};
+	if (const auto& id_doc = priv_->find_document_unlocked(path); !id_doc)
+		return false;
+	else {
+		const Document doc{id_doc->second};
+		for (auto&& label : doc.string_vec_value(Field::Id::Labels))
+			priv_->labels_cache_.decrease(label);
 
-	xapian_db().delete_document(term);
-	mu_debug("deleted message @ {} from store", path);
-	return true;
+		xapian_db().delete_document(term);
+		mu_debug("deleted message @ {} from store", path);
+
+		return true;
+	}
 }
 
 void
@@ -384,13 +430,19 @@ Store::remove_messages(const std::vector<Store::Id>& ids)
 
 	xapian_db().request_transaction();
 
-	for (auto&& id : ids)
-		xapian_db().delete_document(id);
+	for (auto&& id : ids) {
+		if (const auto msg = priv_->find_message_unlocked(id); !msg) {
+			mu_warning("cannot find document {} for deletion", id);
+		} else {
+			for (auto&& label: msg->labels())
+				priv_->labels_cache_.decrease(label);
+			xapian_db().delete_document(id);
+		}
+	}
 
 	xapian_db().request_commit(true/*force*/);
 }
 
-
 Option<Message>
 Store::find_message(Store::Id docid) const
 {
@@ -402,20 +454,14 @@ Store::find_message(Store::Id docid) const
 Option<Store::Id>
 Store::find_message_id(const std::string& path) const
 {
-	constexpr auto path_field{field_from_id(Field::Id::Path)};
-
 	std::lock_guard guard{priv_->lock_};
 
-	auto enq{xapian_db().enquire()};
-	enq.set_query(Xapian::Query{path_field.xapian_term(path)});
-
-	if (auto mset{enq.get_mset(0, 1)}; mset.empty())
+	if (const auto id_doc = priv_->find_document_unlocked(path); !id_doc)
 		return Nothing; // message not found
 	else
-		return Some(*mset.begin());
+		return Some(std::move(id_doc->first));
 }
 
-
 Store::IdMessageVec
 Store::find_messages(IdVec ids) const
 {
@@ -473,7 +519,9 @@ Store::Private::move_message_unlocked(Message&& msg,
 			return Err(res.error());
 
 		/* 4. update message worked; re-store it */
-		if (auto&& res = update_message_unlocked(msg, old_path); !res)
+		if (auto&& res = xapian_db_.replace_document(
+			    field_from_id(Field::Id::Path).xapian_term(old_path),
+			    msg.document().xapian_document()); !res)
 			return Err(res.error());
 	}
 
@@ -589,6 +637,66 @@ Store::contains_message(const std::string& path) const
 	return xapian_db().term_exists(field_from_id(Field::Id::Path).xapian_term(path));
 }
 
+
+Result<Labels::DeltaLabelVec>
+Store::update_labels(Message& msg, const Labels::DeltaLabelVec& labels_delta)
+{
+	if (msg.docid() == Store::InvalidId)
+		return Err(Error::Code::Store, "invalid doc-id");
+
+	std::unique_lock lock{priv_->lock_};
+	// i.e. the set of effective labels. and the set up updates, the "diff"
+	auto updates{updated_labels(msg.labels(), labels_delta)};
+
+	if (updates.second.empty())
+		return Ok(std::move(updates.second)); // nothing to do
+
+	msg.set_labels(updates.first);
+	const auto res = xapian_db().replace_document(msg.xapian_term(),
+						      msg.document().xapian_document());
+	if (!res)
+		return Err(res.error());
+
+	priv_->labels_cache_.update(updates.second);
+
+	return Ok(std::move(updates.second));
+}
+
+Result<Labels::DeltaLabelVec>
+Store::clear_labels(Message& msg)
+{
+	using namespace Labels;
+
+	DeltaLabelVec updates;
+
+	{
+		std::unique_lock lock{priv_->lock_};
+		for (auto& label: msg.labels())
+			updates.emplace_back(DeltaLabel{Delta::Remove, std::move(label)});
+	}
+
+	return update_labels(msg, updates);
+}
+
+Result<void>
+Store::restore_label_map()
+{
+	if (xapian_db().read_only())
+		throw Error{Error::Code::Store, "store is read-only"};
+
+	std::unique_lock lock{priv_->lock_};
+
+	return priv_->labels_cache_.restore(*this);
+}
+
+LabelsCache::Map
+Store::label_map() const
+{
+	std::unique_lock lock{priv_->lock_};
+
+	return priv_->labels_cache_.label_map();
+}
+
 std::size_t
 Store::for_each_message_path(Store::ForEachMessageFunc msg_func) const
 {
