@@ -51,12 +51,6 @@ XapianDb::read_only() const
 	return !std::holds_alternative<Xapian::WritableDatabase>(db_);
 }
 
-const std::string&
-XapianDb::path() const
-{
-	return path_;
-}
-
 void
 XapianDb::set_timestamp(const std::string_view key)
 {
@@ -102,8 +96,7 @@ make_db(const std::string& db_path, Flavor flavor)
 XapianDb::XapianDb(const std::string& db_path, Flavor flavor):
 	path_(make_path(db_path, flavor)),
 	db_(make_db(path_, flavor)),
-	batch_size_{Config(*this).get<Config::Id::BatchSize>()} // default
-{
+	batch_size_{Config(*this).get<Config::Id::BatchSize>()/*default*/} {
 	if (flavor == Flavor::CreateOverwrite)
 		set_timestamp(MetadataIface::created_key);
 
