@@ -1,5 +1,5 @@
  /*
-** Copyright (C) 2008-2024 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2025 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -383,7 +383,10 @@ output_json(const Option<Message>& msg, const OutputInfo& info, const Options& o
 	if (!msg)
 		return Ok();
 
-	mu_println("{}{}", msg->sexp().to_json_string(), info.last ? "" : ",");
+	const Sexp::Format frm{opts.find.format == Format::Json2 ?  Sexp::Format::NoColon :
+		Sexp::Format::Default};
+
+	mu_println("{}{}", msg->sexp().to_json_string(frm), info.last ? "" : ",");
 
 	return Ok();
 }
@@ -443,6 +446,7 @@ get_output_func(const Options& opts)
 	case Format::Sexp:
 		return output_sexp;
 	case Format::Json:
+	case Format::Json2:
 		return output_json;
 	default:
 		throw Error(Error::Code::Internal,
@@ -529,14 +533,6 @@ Mu::mu_cmd_find(const Store& store, const Options& opts)
 
 static std::string test_mu_home;
 
-auto count_nl(const std::string& s)->size_t {
-	size_t n{};
-	for (auto&& c: s)
-		if (c == '\n')
-			++n;
-	return n;
-}
-
 static size_t
 search_func(const std::string& expr, size_t expected)
 {
