@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2022-2024 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2022-2025 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -33,7 +33,6 @@
  *
  */
 
-
 #include <config.h>
 #include <stdexcept>
 #include <array>
@@ -49,17 +48,18 @@
 #include "mu-options.hh"
 #include "mu-script.hh"
 
-#include <thirdparty/CLI11.hpp>
+#ifdef USE_EMBEDDED_CLI11
+#include "CLI11.hpp"
+#else
+#include "CLI/CLI.hpp"
+#endif
 
 using namespace Mu;
 
-
 /*
  * helpers
  */
 
-
-
 /**
  * array of associated pair elements -- like an alist
  * but based on std::array and thus can be constexpr
@@ -67,7 +67,6 @@ using namespace Mu;
 template<typename T1, typename T2, std::size_t N>
       using AssocPairs = std::array<std::pair<T1, T2>, N>;
 
-
 /**
   * Get the first value of the pair where the second element is @param s.
   *
@@ -104,7 +103,6 @@ to_second(const P& p, typename P::value_type::first_type f)
 	return Nothing;
 }
 
-
 /**
  * Options-specific array-bases type that maps some enum to a <name, description> pair
  */
@@ -169,7 +167,6 @@ options_help(const IE& ie, typename IE::value_type::first_type default_opt)
 	return s;
 }
 
-
 /**
  * Get map from string->type
  */
@@ -213,6 +210,15 @@ sub_crypto(CLI::App& sub, T& opts)
 		     "Attempt to decrypt");
 }
 
+static void add_muhome_option(CLI::App& sub, Options& opts)
+{
+	sub.add_option("--muhome",
+		       opts.muhome, "Specify alternative mu directory")
+		->envname("MUHOME")
+		->type_name("<dir>")
+		->transform(ExpandPath, "expand muhome path");
+}
+
 /*
  * subcommands
  */
@@ -265,7 +271,6 @@ sub_cfind(CLI::App& sub, Options& opts)
 }
 
 
-
 static void
 sub_extract(CLI::App& sub, Options& opts)
 {
@@ -318,7 +323,6 @@ sub_fields(CLI::App& sub, Options& opts)
 	// nothing to do.
 }
 
-
 static void
 sub_find(CLI::App& sub, Options& opts)
 {
@@ -339,6 +343,9 @@ sub_find(CLI::App& sub, Options& opts)
 			{ Format::Json,
 			  {"json", "JSON"}
 			},
+			{ Format::Json2,
+			  {"json2", "more idiomatic JSON"}
+			},
 		}};
 
 	sub.add_flag("--threads,-t", opts.find.threads,
@@ -434,7 +441,6 @@ sub_index(CLI::App& sub, Options& opts)
 		     "Perform a complete reindexing");
 }
 
-
 static void
 sub_info(CLI::App& sub, Options& opts)
 {
@@ -462,8 +468,9 @@ sub_init(CLI::App& sub, Options& opts)
 		->transform(ExpandPath, "expand maildir path");
 	// don't attempt to canonicalize; in bash this breaks together with
 	// expand path.
-	sub.add_option("--my-address", opts.init.my_addresses,
-		       "Personal e-mail address or regexp")
+	sub.add_option("--personal-address,--my-address",
+		       opts.init.personal_addresses,
+		       "Personal e-mail address or regexp (can be used multiple titmes)")
 		->type_name("<address>");
 	sub.add_option("--ignored-address", opts.init.ignored_addresses,
 		       "Ignored e-mail address or regexp")
@@ -478,6 +485,7 @@ sub_init(CLI::App& sub, Options& opts)
 	sub.add_flag("--reinit", opts.init.reinit,
 		       "Re-initialize database with current settings")
 		->excludes("--maildir")
+		->excludes("--personal-address")
 		->excludes("--my-address")
 		->excludes("--ignored-address")
 		->excludes("--max-message-size")
@@ -485,6 +493,85 @@ sub_init(CLI::App& sub, Options& opts)
 		->excludes("--support-ngrams");
 }
 
+static void
+sub_labels(CLI::App& sub, Options& opts)
+{
+	sub.require_subcommand(0);
+
+	// update
+	auto update{sub.add_subcommand("update", "update labels")};
+	update->add_option("--labels", opts.labels.delta_labels,
+		       "One or more comma-separated +label,-label")
+		->delimiter(',')
+		->type_name("<delta-label>")
+		->required();
+	update->add_flag("-n,--dry-run", opts.labels.dry_run,
+		     "Output what would change without changing anything");
+	update->add_option("query", opts.labels.query, "Query for messages to update")
+		->required();
+	add_muhome_option(*update, opts);
+
+	// clear
+	auto clear = sub.add_subcommand("clear", "clear all labels from matched messages");
+	clear ->add_option("query", opts.labels.query, "Query for messages to clear of labels")
+		->required();
+	clear->add_flag("-n,--dry-run", opts.labels.dry_run,
+		     "Output what would change without changing anything");
+	add_muhome_option(*clear, opts);
+
+	// list
+	[[maybe_unused]] auto list = sub.add_subcommand("list", "list labels in the store");
+	add_muhome_option(*list, opts);
+
+	// restore-list
+	[[maybe_unused]] auto restore_list = sub.add_subcommand(
+		"restore-list", "restore the labels cache");
+	add_muhome_option(*restore_list, opts);
+
+	// export
+	[[maybe_unused]] auto exportsub = sub.add_subcommand("export", "export labels to a file");
+	add_muhome_option(*exportsub, opts);
+	exportsub->add_option("output", opts.labels.file, "File to export labels to")
+		->type_name("<file>");
+
+	// import
+	auto importsub = sub.add_subcommand("import", "import labels from a file");
+	importsub->add_flag("-n,--dry-run", opts.labels.dry_run,
+			    "Output what would change without changing anything");
+	importsub->add_option("input", opts.labels.file, "File with labels to import")
+		->required()
+		->type_name("<file>");
+	add_muhome_option(*importsub, opts);
+
+	// XXX: it'd be nice to make "update" the default command, such that
+	//   mu label foo --labels +a,-b
+	// would be interpreted as
+	//   mu label update foo --labels +a,-b
+	// but no succeeded yet; CLI11 treats 'foo' as unknown sub-command
+
+	sub.final_callback([&](){
+		if (sub.got_subcommand("list")) {
+			opts.labels.sub = Options::Labels::Sub::List;
+			opts.labels.read_only = true;
+		} else if (sub.got_subcommand("restore-list")) {
+			opts.labels.sub = Options::Labels::Sub::RestoreList;
+			opts.labels.read_only  = false;/*opts.labels.dry_run*/
+		} else if (sub.got_subcommand("clear")) {
+			opts.labels.sub = Options::Labels::Sub::Clear;
+			opts.labels.read_only = opts.labels.dry_run;
+		} else if (sub.got_subcommand("update")){
+			opts.labels.sub = Options::Labels::Sub::Update;
+			opts.labels.read_only = opts.labels.dry_run;
+		} else if (sub.got_subcommand("export")){
+			opts.labels.sub = Options::Labels::Sub::Export;
+			opts.labels.read_only = true;
+		} else if (sub.got_subcommand("import")){
+			opts.labels.sub = Options::Labels::Sub::Import;
+			opts.labels.read_only = opts.labels.dry_run;
+		}
+	});
+}
+
 static void
 sub_mkdir(CLI::App& sub, Options& opts)
 {
@@ -497,7 +584,6 @@ sub_mkdir(CLI::App& sub, Options& opts)
 		->required();
 }
 
-
 static void
 sub_move(CLI::App& sub, Options& opts)
 {
@@ -520,7 +606,6 @@ sub_move(CLI::App& sub, Options& opts)
 		->type_name("<maildir>");
 }
 
-
 static void
 sub_remove(CLI::App& sub, Options& opts)
 {
@@ -529,6 +614,24 @@ sub_remove(CLI::App& sub, Options& opts)
 		->type_name("<files>");
 }
 
+static std::string
+concoct_socket_path()
+{
+	return join_paths(g_get_user_runtime_dir(),
+			  mu_format("mu-scm-{}.sock", ::getpid()));
+}
+
+static void
+add_listen_flag(CLI::App& sub, Options& opts)
+{
+	sub.add_flag("--listen", opts.scm.listen,
+		     "Start SCM REPL on a domain socket");
+	sub.callback([&opts]{
+		if (opts.scm.listen)
+			opts.scm.socket_path = concoct_socket_path();
+	});
+}
+
 static void
 sub_server(CLI::App& sub, Options& opts)
 {
@@ -541,6 +644,22 @@ sub_server(CLI::App& sub, Options& opts)
 		     "Allow for the temp-file optimization")
 		->excludes("--commands");
 
+#if BUILD_SCM
+	add_listen_flag(sub, opts);
+#endif/*BUILD_SCM*/
+}
+
+static void
+sub_scm(CLI::App& sub, Options& opts)
+{
+	add_listen_flag(sub, opts);
+
+	sub.add_option("script-path", opts.scm.script_path, "Path to script")
+		->type_name("<path>")
+		->excludes("--listen");
+	sub.add_option("script-args", opts.scm.params, "Parameters for script")
+		->type_name("<parameters>");
+
 }
 
 static void
@@ -596,7 +715,6 @@ sub_view(CLI::App& sub, Options& opts)
 		->type_name("<message-path>");
 }
 
-
 using	SubCommand = Options::SubCommand;
 using   Category   = Options::Category;
 
@@ -614,15 +732,15 @@ static constexpr
 AssocPairs<SubCommand, CommandInfo, Options::SubCommandNum> SubCommandInfos= {{
 		{ SubCommand::Add,
 		  { Category::NeedsWritableStore,
-		    "add", "Add message(s) to the database", sub_add}
+		    "add", "Add messages to the database", sub_add}
 		},
 		{ SubCommand::Cfind,
 		  { Category::NeedsReadOnlyStore,
-		    "cfind", "Find contacts matching pattern", sub_cfind}
+		    "cfind", "Find contacts matching some pattern", sub_cfind}
 		},
 		{ SubCommand::Extract,
 		  {Category::None,
-		  "extract", "Extract MIME-parts from messages", sub_extract}
+		  "extract", "Extract attachements and other MIME-parts", sub_extract}
 		},
 		{ SubCommand::Fields,
 		  {Category::None,
@@ -630,7 +748,7 @@ AssocPairs<SubCommand, CommandInfo, Options::SubCommandNum> SubCommandInfos= {{
 		},
 		{ SubCommand::Find,
 		  {Category::NeedsReadOnlyStore,
-		  "find", "Find messages matching query", sub_find }
+		  "find", "Find messages matching some query", sub_find }
 		},
 		{ SubCommand::Help,
 		  {Category::None,
@@ -638,15 +756,19 @@ AssocPairs<SubCommand, CommandInfo, Options::SubCommandNum> SubCommandInfos= {{
 		},
 		{ SubCommand::Index,
 		  {Category::NeedsWritableStore,
-		  "index", "Store message information in the database", sub_index }
+		  "index", "Scan maildirs and store information", sub_index }
 		},
 		{ SubCommand::Info,
 		  {Category::NeedsReadOnlyStore,
-		  "info", "Show information", sub_info }
+		  "info", "Show information about mu", sub_info }
 		},
 		{ SubCommand::Init,
 		  {Category::NeedsWritableStore,
-		  "init", "Initialize the database", sub_init }
+		  "init", "Initialize the mu database", sub_init }
+		},
+		{ SubCommand::Labels,
+		  {Category::None, // note Store handled on sub-subcommmands
+		  "labels", "Manage message labels", sub_labels }
 		},
 		{ SubCommand::Mkdir,
 		  {Category::None,
@@ -654,15 +776,19 @@ AssocPairs<SubCommand, CommandInfo, Options::SubCommandNum> SubCommandInfos= {{
 		},
 		{ SubCommand::Move,
 		  {Category::NeedsWritableStore,
-		   "move", "Move a message or change flags", sub_move }
+		   "move", "Move a message or change its flags", sub_move }
 		},
 		{ SubCommand::Remove,
 		  {Category::NeedsWritableStore,
 		  "remove", "Remove message from file-system and database", sub_remove }
 		},
+		{ SubCommand::Scm,
+		  {Category::NeedsReadOnlyStore,
+		   "scm", "Start Guile/Scheme shell or run script",sub_scm}
+		},
 		{ SubCommand::Script,
 		  // Note: SubCommand::Script is special; there's no literal
-		// "script" subcommand, there subcommands for all the scripts.
+		  // "script" subcommand, there are subcommands for all the scripts.
 		  {Category::None,
 		   "script", "Invoke a script", {}}
 		},
@@ -680,8 +806,6 @@ AssocPairs<SubCommand, CommandInfo, Options::SubCommandNum> SubCommandInfos= {{
 		},
 	}};
 
-
-
 static ScriptInfos
 add_scripts(CLI::App& app, Options& opts)
 {
@@ -702,7 +826,6 @@ add_scripts(CLI::App& app, Options& opts)
 #endif /*BUILD_GUILE*/
 }
 
-
 static Result<Options>
 show_manpage(Options& opts, const std::string& name)
 {
@@ -721,7 +844,6 @@ show_manpage(Options& opts, const std::string& name)
 	return Ok(std::move(opts));
 }
 
-
 static Result<Options>
 cmd_help(const CLI::App& app, Options& opts)
 {
@@ -775,17 +897,17 @@ Result<Options>
 Options::make(int argc, char *argv[])
 {
 	Options opts{};
-	CLI::App app{"mu mail indexer/searcher", "mu"};
+	CLI::App app{"mu mail indexer/searcher " PACKAGE_VERSION, "mu"};
 
 	app.description(R"(mu mail indexer/searcher
-Copyright (C) 2008-2023 Dirk-Jan C. Binnema
+Copyright (C) 2008-2025 Dirk-Jan C. Binnema
 
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.
 )");
 	app.set_version_flag("-V,--version", PACKAGE_VERSION);
-	app.set_help_flag("-h,--help", "Show help informmation");
+	app.set_help_flag("-h,--help", "Show help information");
 	app.set_help_all_flag("--help-all");
 	app.require_subcommand(0, 1);
 
@@ -818,11 +940,7 @@ There is NO WARRANTY, to the extent permitted by law.
 		/* store commands get the '--muhome' parameter as well */
 		if (cat == Category::NeedsReadOnlyStore ||
 		    cat == Category::NeedsWritableStore)
-			sub->add_option("--muhome",
-					opts.muhome, "Specify alternative mu directory")
-				->envname("MUHOME")
-				->type_name("<dir>")
-				->transform(ExpandPath, "expand muhome path");
+			add_muhome_option(*sub, opts);
 	}
 
 	/* add scripts (if supported) as semi-subcommands as well */
@@ -898,7 +1016,6 @@ validate_subcommand_ids()
 	return true;
 }
 
-
 /*
  * tests... also build as runtime-tests, so we can get coverage info
  */
@@ -906,7 +1023,6 @@ validate_subcommand_ids()
 #define static_assert g_assert_true
 #endif /*BUILD_TESTS*/
 
-
 [[maybe_unused]]
 static void
 test_ids()
