@@ -1,6 +1,6 @@
-;;; mu4e-vars.el -- part of mu4e, the mu mail user agent -*- lexical-binding: t -*-
+;;; mu4e-vars.el --- Variables and faces for mu4e -*- lexical-binding: t -*-
 
-;; Copyright (C) 2011-2023 Dirk-Jan C. Binnema
+;; Copyright (C) 2011-2025 Dirk-Jan C. Binnema
 
 ;; Author: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 ;; Maintainer: Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
@@ -26,7 +26,7 @@
 
 (require 'message)
 (require 'mu4e-helpers)
-
+
 ;;; Configuration
 (defgroup mu4e nil
   "Mu4e - an email-client for Emacs."
@@ -75,7 +75,6 @@ and show some \"Loading\" banner."
   :type 'boolean
   :group 'mu4e)
 
-
 ;;; Faces
 
 (defgroup mu4e-faces nil
@@ -268,6 +267,11 @@ Foo\")."
         :shortname "Maildir"
         :help "Maildir for this message"
         :sortable t))
+    (:labels
+     . (:name "Labels"
+        :shortname "Lbs"
+        :help "Labels for the message"
+        :sortable nil))
     (:list
      . (:name "List-Id"
         :shortname "List"
@@ -302,7 +306,8 @@ Foo\")."
      . (:name "Tags"
         :shortname "Tags"
         :help "Tags for the message"
-        :sortable nil))
+        ;; sort by _first_ tag.
+        :sortable t))
     (:thread-subject
      . (:name "Subject"
         :shortname "Subject"
@@ -313,9 +318,11 @@ Foo\")."
         :shortname "To"
         :help "Recipient of the message"
         :sortable t)))
+
   "An alist of all possible header fields and information about them.
-This is used in the user-interface (the column headers in the header list, and
-the fields the message view).
+
+This is used in the user-interface (the column headers in the
+header list, and the fields the message view).
 
 Most fields should be self-explanatory. A special one is
 `:from-or-to', which is equal to `:from' unless `:from' matches
@@ -362,7 +369,7 @@ Note, `:sortable' is not supported for custom header fields.")
                  (+ (length (mu4e-message-field msg :to))
                     (length (mu4e-message-field msg :cc))))))))
 
-  "A list of custom (user-defined) headers.
+  "An  alist of custom (user-defined) headers.
 The format is similar to `mu4e-header-info', but adds a :function
 property, which should point to a function that takes a message
 plist as argument, and returns a string. See the default value of
